from _typeshed import Incomplete

config: Incomplete
DRY_RUN: Incomplete
THREAD_NUM: Incomplete
auth_method: Incomplete
profile_name: Incomplete
access_key_id: Incomplete
secret_access_key: Incomplete
sso_url: Incomplete
sso_role_name: Incomplete
sso_account_id: Incomplete
accounts_list_src: Incomplete
region: Incomplete
assumed_role_name: Incomplete
external_id: Incomplete
sso_instance_arn: Incomplete
identity_store_id: Incomplete
accounts: Incomplete
ou: Incomplete
statuses: Incomplete
actions_get_output_file: Incomplete
actions_get_service: Incomplete
actions_get_filter: Incomplete
sgs_get_output_file: Incomplete
sso_get_output_file: Incomplete
sso_json_input_file: Incomplete
sso_json_output_file: Incomplete
sso_mappings_map_file: Incomplete
sso_mappings_input_file: Incomplete
sso_mappings_output_file: Incomplete
sso_mappings_old_user_heading: Incomplete
sso_mappings_new_user_heading: Incomplete
sso_mappings_old_group_pattern: Incomplete
sso_mappings_new_group_pattern: Incomplete
sso_assign_input_file: Incomplete
sso_update_user_map_file: Incomplete
sso_update_group_map_file: Incomplete
sso_update_old_user_heading: Incomplete
sso_update_new_user_heading: Incomplete
sso_update_old_group_heading: Incomplete
sso_update_new_group_heading: Incomplete
sso_memberships_input_file: Incomplete
sso_memberships_output_file: Incomplete
