def read_csv(file_name: str) -> list: ...
def dicts_to_csv(data: list) -> list: ...
def export_csv(data: list, file: str) -> None: ...
def read_json(file_name: str) -> dict: ...
def write_json_obj(file_name: str, data: dict | list) -> None: ...
def write_json_str(file_name: str, data: str) -> None: ...
def main() -> None: ...
